import { Class, Dictionary } from '@agape/types';
import { Choice } from "./types";
export type FieldDescriptorParams = Partial<Pick<FieldDescriptor, keyof FieldDescriptor>>;
export type ModelDescriptorParams = Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>;
/**
 * Meta data describing a model.
 */
export declare class ModelDescriptor {
    name: string;
    label: string;
    plural: string;
    description: string;
    symbol: string;
    token: string;
    tokens: string;
    fields: FieldDescriptorSet;
    readable: boolean;
    writable: boolean;
    constructor(name?: string, params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    constructor(params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    field(name: string): FieldDescriptor;
    add(field: FieldDescriptor): void;
    autopopulate(): void;
}
/**
 * Meta data describing a view
 */
export declare class ViewDescriptor extends ModelDescriptor {
    title?: string;
    model?: Class;
    viewName: string;
    constructor(model?: Class);
    constructor(name?: string, params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
    constructor(params?: Partial<Pick<ModelDescriptor, keyof ModelDescriptor>>);
}
export declare class FieldDescriptor {
    name?: string;
    label?: string;
    plural?: string;
    description?: string;
    length?: number;
    link?: string;
    column?: string;
    required?: boolean;
    token?: string;
    tokens?: string;
    type?: string;
    widget?: string;
    readable?: boolean;
    example?: string | number | Date | Dictionary;
    primary?: boolean;
    choices?: Choice[];
    collection?: string;
    designType: 'string' | 'number' | 'boolean' | 'any' | Class;
    constructor();
    constructor(name: string, type?: string, widget?: string);
    constructor(params: FieldDescriptorParams);
    assign(params: FieldDescriptorParams): void;
    autopopulate(): void;
}
/**
 * A set containing the managed properties that exist on an object.
 */
export declare class FieldDescriptorSet {
    private ʘ;
    constructor(from?: FieldDescriptorSet);
    get length(): number;
    all(): FieldDescriptor[];
    /**
     * Merge descriptors from another set into this set
     * @param from
     */
    merge(from: FieldDescriptorSet): void;
    /**
     * Does a descriptor with the given name exist in the set
     * @param name
     */
    has(name: string): boolean;
    /**
     * Get the descriptor with the provided name
     * @param name
     */
    get(name: string): FieldDescriptor;
    /**
     * Get names of all descriptors which exist in the set.
     */
    get names(): Array<string>;
    /**
     * Set the descriptor of the given name
     * @param name
     * @param descriptor
     */
    set(name: string, descriptor: FieldDescriptor): FieldDescriptor;
    /**
     * Execute a function on each item in the set
     * @param callback
     */
    forEach(callback: (propertyName: string, definition: FieldDescriptor) => void): void;
}
