"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FieldDescriptorSet = exports.FieldDescriptor = exports.ViewDescriptor = exports.ModelDescriptor = void 0;
const string_1 = require("@agape/string");
/**
 * Meta data describing a model.
 */
class ModelDescriptor {
    constructor(...args) {
        this.fields = new FieldDescriptorSet();
        let name;
        let params;
        if (args.length === 1 && args[0] instanceof Object) {
            params = Object.assign({}, args[0]);
        }
        else {
            [name, params] = args;
            params !== null && params !== void 0 ? params : (params = {});
            params.name = name;
        }
        if (params)
            Object.assign(this, params);
        this.autopopulate();
    }
    field(name) {
        if (!this.fields.has(name)) {
            const descriptor = new FieldDescriptor(name);
            this.fields.set(name, descriptor);
            return descriptor;
        }
        return this.fields.get(name);
    }
    add(field) {
        if (this.fields.has(field.name)) {
            throw new Error(`Field ${field.name} is already defined on ${this.name}`);
        }
        this.fields.set(field.name, field);
    }
    autopopulate() {
        var _a, _b, _c, _d, _e;
        if (this.name || this.label) {
            (_a = this.name) !== null && _a !== void 0 ? _a : (this.name = (0, string_1.camelize)(this.label));
            (_b = this.label) !== null && _b !== void 0 ? _b : (this.label = (0, string_1.verbalize)(this.name));
            (_c = this.plural) !== null && _c !== void 0 ? _c : (this.plural = (0, string_1.pluralize)(this.label));
            (_d = this.token) !== null && _d !== void 0 ? _d : (this.token = (0, string_1.tokenize)(this.name));
            (_e = this.tokens) !== null && _e !== void 0 ? _e : (this.tokens = (0, string_1.pluralize)(this.token));
        }
    }
}
exports.ModelDescriptor = ModelDescriptor;
/**
 * Meta data describing a view
 */
class ViewDescriptor extends ModelDescriptor {
    constructor(...args) {
        if (args.length === 1) {
            const model = args.shift();
            super();
            this.model = model;
        }
        else {
            super(...args);
        }
    }
}
exports.ViewDescriptor = ViewDescriptor;
class FieldDescriptor {
    constructor(...args) {
        let params;
        if (args.length === 1 && !(typeof args[0] === 'string')) {
            params = Object.assign({}, args[0]);
        }
        else {
            const [name, type, widget] = args;
            params = { name, type, widget };
        }
        this.assign(params);
    }
    assign(params) {
        Object.assign(this, params);
    }
    autopopulate() {
        var _a, _b, _c, _d, _e;
        if (this.name || this.label) {
            (_a = this.name) !== null && _a !== void 0 ? _a : (this.name = (0, string_1.camelize)(this.label));
            (_b = this.label) !== null && _b !== void 0 ? _b : (this.label = (0, string_1.verbalize)(this.name));
            (_c = this.plural) !== null && _c !== void 0 ? _c : (this.plural = (0, string_1.pluralize)(this.label));
            (_d = this.token) !== null && _d !== void 0 ? _d : (this.token = (0, string_1.tokenize)(this.name));
            (_e = this.tokens) !== null && _e !== void 0 ? _e : (this.tokens = (0, string_1.pluralize)(this.token));
        }
    }
}
exports.FieldDescriptor = FieldDescriptor;
/**
 * A set containing the managed properties that exist on an object.
 */
class FieldDescriptorSet {
    constructor(from) {
        this.ʘ = {};
        if (from)
            this.merge(from);
    }
    get length() {
        return Object.keys(this.ʘ).length;
    }
    all() {
        return Object.values(this.ʘ);
    }
    /**
     * Merge descriptors from another set into this set
     * @param from
     */
    merge(from) {
        Object.assign(this.ʘ, from.ʘ);
    }
    /**
     * Does a descriptor with the given name exist in the set
     * @param name
     */
    has(name) {
        return name in this.ʘ;
    }
    /**
     * Get the descriptor with the provided name
     * @param name
     */
    get(name) {
        return this.ʘ[name];
    }
    /**
     * Get names of all descriptors which exist in the set.
     */
    get names() {
        return Object.getOwnPropertyNames(this.ʘ);
    }
    /**
     * Set the descriptor of the given name
     * @param name
     * @param descriptor
     */
    set(name, descriptor) {
        return this.ʘ[name] = descriptor;
    }
    /**
     * Execute a function on each item in the set
     * @param callback
     */
    forEach(callback) {
        for (let propertyName in this.ʘ) {
            callback(propertyName, this.ʘ[propertyName]);
        }
    }
}
exports.FieldDescriptorSet = FieldDescriptorSet;
//# sourceMappingURL=descriptors.js.map